import 'dart:convert';
import 'package:http/http.dart' as http;

import '../vk.dart';
import 'messages.dart';
import 'groups.dart';
import 'utils.dart';
import 'account.dart';

import 'apiexception.dart';

/// VK API class
class API {
  /// Base API request url
  String _baseUrl = 'https://api.vk.com/method';

  /// VK class
  VK _vk;

  /// Messages class
  Messages messages;

  /// Groups class
  Groups groups;

  /// Utils class
  Utils utils;

  /// Account class
  Account account;

  /// API
  API(VK vk) {
    this._vk = vk;

    this.messages = new Messages(this);
    this.groups = new Groups(this);
    this.utils = new Utils(this);
    this.account = new Account(this);
  }

  toString() {
    return 'API';
  }

  Future<Map> request(String method, [Map<String, String> params]) async {
    Map body = <String, String>{
      'access_token': this._vk.options['token'],
      'v': this._vk.options['version'],
      ...?params,
    };

    http.Response response = await http.post(
      this._baseUrl + '/$method',
      body: body,
    );

    Map json = jsonDecode(response.body);

    if (json['error'] != null) {
      Map error = json['error'];

      throw new APIException(
        error['error_msg'],
        code: error['error_code'],
      ).error;
    }

    return json;
  }

  /// Call an API [method] with [params]
  Future<Map> call(String method, [Map<String, String> params]) async {
    return this.request(method, {
      ...?params,
    });
  }

  /// Execute [code]
  Future<Map> execute(String code) async {
    return this.call('execute', {
      'code': code,
    });
  }
}
